/*
 * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */


/*
 * @test
 *
 * @summary converted from VM Testbase nsk/jdi/VirtualMachine/redefineClasses/redefineclasses003.
 * VM Testbase keywords: [quick, jpda, jdi, redefine]
 * VM Testbase readme:
 * DESCRIPTION:
 *     The test against the method com.sun.jdi.VirtualMachine.redefineClasses()
 *     and checks up the following assertion:
 *         "...redefining a class does not cause its initializers to be run.
 *          The values of preexisting static variables will remain as they
 *          were prior to the call."
 *     The test consists of the following files:
 *         redefineclasses003.java           - debugger
 *         redefineclasses003a.java          - initial debuggee
 *         newclass/redefineclasses003a.java - redefining debuggee
 *     The redefined debugee differs from the initial debuggee by static
 *     initializer only.
 *     This test performs the following cases:
 *         1. Values of static fields are assigned by <clinit>
 *         2. Values of static fields are changed by debugger
 *         3. Values of static fields are changed by debugee
 *     After changing values of static fields, debugger redefines <clinit>
 *     and checks that no initializer are run.
 * COMMENTS:
 *     Test was fixed according to test bug:
 *     4778296 TEST_BUG: debuggee VM intemittently hangs after resuming
 *     - handling VMStartEvent was removed from the debugger part of the test
 *
 * @library /vmTestbase
 *          /test/lib
 * @build nsk.jdi.VirtualMachine.redefineClasses.redefineclasses003
 *        nsk.jdi.VirtualMachine.redefineClasses.redefineclasses003a
 *
 * @comment make sure redefineclasses003a is compiled with full debug info
 * @clean nsk.jdi.VirtualMachine.redefineClasses.redefineclasses003a
 * @compile -g:lines,source,vars ../redefineclasses003a.java
 *
 * @comment compile newclassXX to bin/newclassXX
 *          with full debug info
 * @run driver nsk.share.ExtraClassesBuilder
 *      -g:lines,source,vars
 *      newclass
 *
 * @run driver
 *      nsk.jdi.VirtualMachine.redefineClasses.redefineclasses003
 *      ./bin
 *      -verbose
 *      -arch=${os.family}-${os.simpleArch}
 *      -waittime=5
 *      -debugee.vmkind=java
 *      -transport.address=dynamic
 *      -debugee.vmkeys="${test.vm.opts} ${test.java.opts}"
 */

